/*
 * Samples taken from http://radar.oreilly.com/2011/09/java7-features.html
 */
package cz.muni.fi.pa165.examples.currencyconvertor;

import java.io.DataOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import junit.framework.Assert;
import org.junit.Test;

/**
 *
 * @author tomp
 */
public class Java7Test {

    private static final int million = 1_000_000;

    @Test
    public void showStringSwitchTest() {
        Map<String, List<String>> countries = createStringToListOfStrings();
        List<String> firstCz = showStringSwitch("first-cz", countries);
        Assert.assertTrue("Doesn't contain Prague", firstCz.contains("Prague"));
    }

    @Test
    public void oldTry() throws IOException {
        DataOutputStream dos = null;
        try {
            FileOutputStream fos = new FileOutputStream("movies.txt");
            dos = new DataOutputStream(fos);
            dos.writeUTF("Java 7 Block Buster");

            new NullPointerException();
        } catch (IOException e) {
            // log the exception 
            // the streams will NOT be closed
        }

        Assert.assertFalse(new File("movies.txt").delete());
        dos.close();
    }

    @Test
    public void newTry() {
        try (FileOutputStream fos = new FileOutputStream("movies.txt");
                DataOutputStream dos = new DataOutputStream(fos)) {
            dos.writeUTF("Java 7 Block Buster");

            new NullPointerException();
        } catch (IOException e) {
            // log the exception 
            // the streams will automatically be closed
        }
        Assert.assertTrue(new File("movies.txt").delete());
    }

    @Test(expected = BadFormatException.class)
    public void multicatchTest() {
        multiplicativeInverse("asdf");
    }

    @Test(expected = NullOrIllegalException.class)
    public void multicatchTest2() {
        multiplicativeInverse(null);
    }

    @Test(expected = NullOrIllegalException.class)
    public void multicatchTest3() {
        multiplicativeInverse("0");
    }

    private static float multiplicativeInverse(String number) {

        try {
            //This logging line will fail with nullpinter exception when the input is null
            System.out.println("Logging, the size of the input is: " + number.length());
            int result = Integer.parseInt(number);
            return 1 / result;
        } catch (NumberFormatException e) {
            throw new BadFormatException(number + " has bad format for integer");

        } catch (NullPointerException | ArithmeticException e) {
            throw new NullOrIllegalException(number + " is either null or some bad arithemtic problem arised!");
        }
    }

    public static class BadFormatException extends RuntimeException {

        private BadFormatException(String string) {
            super(string);
        }
    }

    public static class NullOrIllegalException extends RuntimeException {

        private NullOrIllegalException(String string) {
            super(string);
        }
    }

    private List<String> showStringSwitch(
            String action,
            Map<String, List<String>> countryToCities) {

        switch (action) {
            case "list-cz":
                return (countryToCities.get("Czech Republic"));
            case "list-sk":
                return (countryToCities.get("Slovakia"));
            case "first-cz":
                return Collections.singletonList(countryToCities.get("Czech Republic").get(0));
            case "first-sk":
                return Collections.singletonList(countryToCities.get("Slovakia").get(0));
            default:
                return countryToCities.get(action);
        }
    }

    private static Map<String, List<String>> createStringToListOfStrings() {
        // this is the new Java 7 generics syntax: 
        // use the diamond instead of full type parameter
        Map<String, List<String>> countryToCities = new TreeMap<>();

        List<String> czCities = Arrays.asList("Prague", "Brno", "Ostrava", "Pilsen");
        countryToCities.put("Czech Republic", czCities);

        List<String> skCities = Arrays.asList("Bratislava", "Košice", "Banská Bystrica");
        countryToCities.put("Slovakia", skCities);

        return countryToCities;
    }
}
